@@ -1,6 +1,7 @@
 #include "math.h" // math first to fix OSX error
 #include "scrimp.h"
 #include "mass.h"
+#include "windowfunc.h"
 #include <numeric>
 // [[Rcpp::depends(RcppProgress)]]
 #include <progress.hpp>
@@ -19,15 +20,13 @@ using namespace RcppParallel;
 
 // [[Rcpp::export]]
 List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, uint32_t window_size, double ez,
-                 double pre_scrimp, bool progress) {
-  // double s_size = R_PosInf;
+                 double s_size, double pre_scrimp, bool progress) {
+
   bool partial = false;
   uint32_t exclusion_zone = round(window_size * ez + DBL_EPSILON);
   uint32_t data_size = data_ref.length();
-  // uint32_t query_size = query_ref.length();
 
   uint32_t matrix_profile_size = data_size - window_size + 1;
-  // uint32_t num_queries = query_size - window_size + 1;
 
   // TODO: check skip position (DBL_EPSILON, etc)
   LogicalVector skip_location(matrix_profile_size, 0);
@@ -66,15 +65,15 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
 
     if (pre_scrimp > 0) {
       // initialization
-      uint64_t current_step = floor(window_size * pre_scrimp + DBL_EPSILON);
+      int64_t current_step = floor(window_size * pre_scrimp + DBL_EPSILON);
       IntegerVector pre_scrimp_idxs = seq_by(0, matrix_profile_size - 1, current_step);
       Progress ps(pre_scrimp_idxs.size(), progress);
       // compute the matrix profile
       NumericVector dotproduct(matrix_profile_size);
       NumericVector refine_distance(matrix_profile_size, R_PosInf);
 
-      uint64_t j = 1;
-      for (uint64_t &&i : pre_scrimp_idxs) {
+      int64_t j = 1;
+      for (int64_t &&i : pre_scrimp_idxs) {
 
         RcppThread::checkUserInterrupt();
         ps.increment();
@@ -85,8 +84,8 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
 
         NumericVector distance_profile = as<NumericVector>(nn["distance_profile"]);
 
-        uint64_t exc_st = 0;
-        uint64_t exc_ed = 0;
+        int64_t exc_st = 0;
+        int64_t exc_ed = 0;
 
         // apply exclusion zone
         if (exclusion_zone > 0) {
@@ -107,24 +106,24 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
         if (j == 1) {
           matrix_profile = distance_profile;
           profile_index.fill(i);
-          uint64_t min_idx = which_min(distance_profile);
+          int64_t min_idx = which_min(distance_profile);
           profile_index[i] = min_idx;
           matrix_profile[i] = distance_profile[min_idx];
         } else {
           LogicalVector update_pos = distance_profile < matrix_profile;
           profile_index[update_pos] = (int32_t)i;
           matrix_profile[update_pos] = distance_profile[update_pos];
-          uint64_t min_idx = which_min(distance_profile);
+          int64_t min_idx = which_min(distance_profile);
           profile_index[i] = min_idx;
           matrix_profile[i] = distance_profile[min_idx];
         }
 
-        uint64_t idx_nn = profile_index[i];
-        int64_t idx_diff = idx_nn - i;
+        int64_t idx_nn = profile_index[i];
+        int64_t idx_diff = (idx_nn - i);
         dotproduct[i] = (window_size - (matrix_profile[i] / 2)) * data_sd[i] * data_sd[idx_nn] +
                         window_size * data_mean[i] * data_mean[idx_nn];
 
-        uint64_t endidx = MIN(matrix_profile_size - 1, (int64_t)i + current_step - 1);
+        int64_t endidx = MIN(matrix_profile_size - 1, (int64_t)i + current_step - 1);
         endidx = MIN((int64_t)endidx, matrix_profile_size - idx_diff - 1);
 
         if (i < endidx) {
@@ -145,9 +144,9 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
                                      (data_sd[dot_idxs1] * data_sd[ref_idxs1]));
         }
 
-        uint64_t beginidx = ((i + 1) <= current_step) ? 0 : (i + 1 - current_step);
+        int64_t beginidx = ((i + 1) <= current_step) ? 0 : (i + 1 - current_step);
         if (idx_diff < 0) {
-          beginidx = MAX(beginidx, (uint64_t)abs(idx_diff));
+          beginidx = MAX(beginidx, abs(idx_diff));
         }
 
         if (i > 0 && i > beginidx) {
@@ -187,7 +186,7 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
         new_idxs = as<IntegerVector>(orig_index[(update_pos2 + beginidx + idx_diff)]) - idx_diff;
         profile_index[(update_pos2 + beginidx + idx_diff)] = new_idxs;
 
-        j = j + 1;
+        j++;
       }
     }
 
@@ -202,8 +201,16 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
 
     Progress p(compute_order.size(), progress);
 
+    compute_order = sample(compute_order, compute_order.size());
+
+    uint64_t stop = 0;
+
+    if (s_size < 1.0) {
+      stop = round(compute_order.size() * s_size + DBL_EPSILON);
+    }
+
     uint64_t j = 1;
-    for (uint64_t &&i : compute_order) {
+    for (int64_t &&i : compute_order) {
 
       RcppThread::checkUserInterrupt();
       p.increment();
@@ -246,6 +253,10 @@ List scrimp_rcpp(const NumericVector data_ref, const NumericVector query_ref, ui
       matrix_profile[loc2] = dist2[loc2];
       profile_index[loc2] = (IntegerVector)(as<IntegerVector>(orig_index[loc2]) + i);
 
+      if (stop > 0 && j >= stop) {
+        partial = true;
+        break;
+      }
       j++;
     }
 
@@ -382,7 +393,7 @@ struct ScrimpWorker : public Worker {
 
 // [[Rcpp::export]]
 List scrimp_rcpp_parallel(const NumericVector data_ref, const NumericVector query_ref, uint32_t window_size, double ez,
-                          bool progress) {
+                          double s_size, bool progress) {
   uint64_t exclusion_zone = round(window_size * ez + DBL_EPSILON);
   uint64_t data_size = data_ref.length();
   // uint64_t query_size = query_ref.length();
@@ -439,3 +450,179 @@ List scrimp_rcpp_parallel(const NumericVector data_ref, const NumericVector quer
                        Rcpp::Named("profile_index") = profile_index + 1, Rcpp::Named("partial") = partial,
                        Rcpp::Named("ez") = ez));
 }
+
+// [[Rcpp::export]]
+List scrimpab_rcpp(const NumericVector data_ref, const NumericVector query_ref, uint32_t window_size, double s_size,
+                   bool progress) {
+  // double s_size = R_PosInf;
+  bool partial = false;
+  uint32_t data_size = data_ref.length();
+  uint32_t query_size = query_ref.length();
+
+  uint32_t mmpa_size = data_size - window_size + 1;
+  uint32_t mmpb_size = query_size - window_size + 1;
+
+  // TODO: check skip position (DBL_EPSILON, etc)
+  LogicalVector skip_location(mmpa_size, 0);
+
+  for (uint64_t i = 0; i < mmpa_size; i++) {
+    NumericVector range = data_ref[Range(i, (i + window_size - 1))];
+    if (any(is_na(range) | is_infinite(range))) {
+      skip_location[i] = TRUE;
+    }
+  }
+
+  NumericVector data = data_ref;
+  NumericVector query = query_ref;
+
+  data[is_na(data)] = 0;
+  data[is_infinite(data)] = 0;
+  query[is_na(query)] = 0;
+  query[is_infinite(query)] = 0;
+
+  NumericVector mmpa(mmpa_size, R_PosInf);
+  IntegerVector mpia(mmpa_size, -1);
+  NumericVector mmpb(mmpb_size, R_PosInf);
+  IntegerVector mpib(mmpb_size, -1);
+
+  IntegerVector orig_index = Range(0, mmpa_size - 1);
+
+  List dd = movmean_std_rcpp(data, window_size);
+  List qq = movmean_std_rcpp(query, window_size);
+
+  NumericVector data_mean = dd["avg"];
+  NumericVector data_sd = dd["sd"];
+  NumericVector query_mean = qq["avg"];
+  NumericVector query_sd = qq["sd"];
+
+  Progress p(orig_index.size() * 2 - 2, progress);
+
+  try {
+    //// SCRIMP ----
+
+    IntegerVector compute_order = orig_index[orig_index > 1];
+
+    NumericVector curlastz(mmpb_size);
+    NumericVector curdistance(mmpb_size);
+    NumericVector dist1(mmpb_size, R_PosInf);
+    NumericVector dist2(mmpb_size, R_PosInf);
+
+    int64_t j = 1;
+    for (int64_t &&i : compute_order) {
+
+      RcppThread::checkUserInterrupt();
+      p.increment();
+
+      curlastz[i] = sum(data[Range(0, window_size - 1)] * query[Range(i, i + window_size - 1)]);
+
+      if (i < (mmpb_size - 1)) {
+        curlastz[Range(i + 1, mmpb_size - 1)] =
+            (NumericVector)cumsum(data[Range(window_size, data_size - i - 1)] *
+                                      query[Range(i + window_size, query_size - 1)] -
+                                  data[Range(0, mmpb_size - i - 2)] * query[Range(i, mmpb_size - 2)]) +
+            curlastz[i];
+      }
+      curdistance[Range(i, mmpb_size - 1)] =
+          2 * (window_size - (curlastz[Range(i, mmpb_size - 1)] - window_size * query_mean[Range(i, mmpb_size - 1)] *
+                                                                      data_mean[Range(0, mmpb_size - i - 1)]) /
+                                 (query_sd[Range(i, mmpb_size - 1)] * data_sd[Range(0, mmpb_size - i - 1)]));
+
+      LogicalVector cd = (curdistance < 0);
+
+      if (sum(as<IntegerVector>(cd)) > 0) {
+        Rcout << "Debug: curdistance < 0" << std::endl;
+      }
+
+      curdistance[cd] = 0;
+
+      dist1[::seq(0, i - 1)] = R_PosInf;
+      dist1[::seq(i, mmpb_size - 1)] = curdistance[::seq(i, mmpb_size - 1)];
+      LogicalVector loc1 = dist1 < mmpb;
+      mmpb[loc1] = dist1[loc1];
+      mpib[loc1] = (IntegerVector)(as<IntegerVector>(orig_index[loc1]) - i);
+
+      dist2[::seq(0, mmpb_size - i - 1)] = curdistance[::seq(i, mmpb_size - 1)];
+      dist2[::seq(mmpb_size - i, mmpb_size - 1)] = R_PosInf;
+      LogicalVector loc2 = dist2 < mmpa;
+      mmpa[loc2] = dist2[loc2];
+      mpia[loc2] = (IntegerVector)(as<IntegerVector>(orig_index[loc2]) + i);
+
+      j++;
+    }
+
+  } catch (RcppThread::UserInterruptException &e) {
+    partial = true;
+    Rcout << "Process terminated by the user successfully, partial results were returned." << std::endl;
+  } catch (...) {
+    ::Rf_error("c++ exception (unknown reason)");
+  }
+
+  try {
+    //// SCRIMP ----
+
+    IntegerVector compute_order = orig_index[orig_index > 1];
+
+    NumericVector curlastz(mmpa_size);
+    NumericVector curdistance(mmpa_size);
+    NumericVector dist1(mmpa_size, R_PosInf);
+    NumericVector dist2(mmpa_size, R_PosInf);
+
+    int64_t j = 1;
+    for (int64_t &&i : compute_order) {
+
+      RcppThread::checkUserInterrupt();
+      p.increment();
+
+      curlastz[i] = sum(query[Range(0, window_size - 1)] * data[Range(i, i + window_size - 1)]);
+
+      if (i < (mmpa_size - 1)) {
+        curlastz[Range(i + 1, mmpa_size - 1)] =
+            (NumericVector)cumsum(query[Range(window_size, data_size - i - 1)] *
+                                      data[Range(i + window_size, query_size - 1)] -
+                                  query[Range(0, mmpa_size - i - 2)] * data[Range(i, mmpa_size - 2)]) +
+            curlastz[i];
+      }
+      curdistance[Range(i, mmpa_size - 1)] =
+          2 * (window_size - (curlastz[Range(i, mmpa_size - 1)] - window_size * data_mean[Range(i, mmpa_size - 1)] *
+                                                                      query_mean[Range(0, mmpa_size - i - 1)]) /
+                                 (data_sd[Range(i, mmpa_size - 1)] * query_sd[Range(0, mmpa_size - i - 1)]));
+
+      LogicalVector cd = (curdistance < 0);
+
+      if (sum(as<IntegerVector>(cd)) > 0) {
+        Rcout << "Debug: curdistance < 0" << std::endl;
+      }
+
+      curdistance[cd] = 0;
+
+      dist1[::seq(0, i - 1)] = R_PosInf;
+      dist1[::seq(i, mmpa_size - 1)] = curdistance[::seq(i, mmpa_size - 1)];
+      LogicalVector loc1 = dist1 < mmpa;
+      mmpa[loc1] = dist1[loc1];
+      mpia[loc1] = (IntegerVector)(as<IntegerVector>(orig_index[loc1]) - i);
+
+      dist2[::seq(0, mmpa_size - i - 1)] = curdistance[::seq(i, mmpa_size - 1)];
+      dist2[::seq(mmpa_size - i, mmpa_size - 1)] = R_PosInf;
+      LogicalVector loc2 = dist2 < mmpb;
+      mmpb[loc2] = dist2[loc2];
+      mpib[loc2] = (IntegerVector)(as<IntegerVector>(orig_index[loc2]) + i);
+
+      j++;
+    }
+
+  } catch (RcppThread::UserInterruptException &e) {
+    partial = true;
+    Rcout << "Process terminated by the user successfully, partial results were returned." << std::endl;
+  } catch (...) {
+    ::Rf_error("c++ exception (unknown reason)");
+  }
+
+  mmpa = sqrt(mmpa);
+  mmpb = sqrt(mmpb);
+  mpia = mpia + 1;
+  mpib = mpib + 1;
+
+  return (List::create(Rcpp::Named("matrix_profile") = mmpa, Rcpp::Named("profile_index") = mpia,
+                       Rcpp::Named("mpb") = mmpb, Rcpp::Named("pib") = mpib, Rcpp::Named("partial") = partial,
+                       Rcpp::Named("ez") = 0));
+}
